"""
Utility to create an argparser with predefined arguments. 
https://docs.python.org/3/library/argparse.html#module-argparse
"""
import argparse
from ._arguments import *

def get_parser(prog, description, *args):
    """
    Return a parser with the specified arguments. Each arg
    in *args must be defined in ARGUMENTS.
    """
    parser = argparse.ArgumentParser(prog=prog, description=description)

    for arg_name in args:
        arg_def = ARGUMENTS[arg_name]

        if arg_def.get("flag"):
            parser.add_argument(arg_name, arg_def.pop("flag"), **arg_def)
        else:
            parser.add_argument(arg_name, **arg_def)

    return parser


if __name__ == "__main__":
    # tests
    name = "fake_program.py"
    description = "Fake program which does nothing useful."

    parser = get_parser(
        name,
        description,
        "dataset",
        "device_type",
        "app_name",
        "eval_type",
        "--destination",
        "--replace",
        "--json",
        "--last_run_date"
    )
    
    print(
        parser.parse_args([
            "cameras",
            "gridsmart",
            "data_tracker_prod",
            "traffic_signal",
            "-d",
            "socrata",
            "--replace",
            "--json",
            "--last_run_date",
            "1535997869"
        ])
    )


ARGUMENTS = {
    "dataset": {
        "action": "store",
        "type": str,
        "help": "Name of the dataset that will be published. Must match entry in Knack config file.",
    },
    "device_type": {
        "action": "store",
        "type": str,
        "choices": [
            "signals",
            "travel_sensors",
            "cameras",
            "cameras_cell_modem",
            "cameras_not_cell_modem",
            "gridsmart",
            "detectors",
            "cabinets",
        ],
        "help": "Type of device to ping.",
    },
    "eval_type": {
        "action": "store",
        "choices": ["phb", "traffic_signal"],
        "type": str,
        "help": "The type of evaluation score to rank.",
    },
    "app_name": {
        "action": "store",
        "choices": [
            "data_tracker_prod",
            "data_tracker_test",
            "visitor_sign_in_prod",
            "dts_portal_test",
            "dts_portal_prod",
            "signs_markings_prod",
            "signs_markings_test"
        ],
        "type": str,
        "help": "Name of the knack application that will be accessed",
    },
    "provider_name": {
        "action": "store",
        "type": str,
        "help": "Name of the mobiltiy provider.",
    },
    "--destination": {
        "flag": "-d",
        "action": "append",
        "choices": ["socrata", "agol", "csv"],
        "required": True,
        "type": str,
        "help": "Destination dataset(s) to which data will be published. Can be repeated for multiple destinations.",
    },
    "--json": {
        "action": "store_true",
        "default": False,
        "help": "Write device data to JSON.",
    },
    "--replace": {
        "flag": "-r",
        "action": "store_true",
        "default": False,
        "help": "Replace all destination data with source data.",
    },
    "--last_run_date": {
        "flag": "-l",
        "action": "store",
        "type": int,
        "help": "A unix timestamp representing the last date the job was run. Will be applied as a temporal filter when querying data for processing.",
    },
    "--start": {
        "action": "store",
        "type": int,
        "help": "A unix timestamp representing a start datetime.",
    },
    "--end": {
        "action": "store",
        "type": int,
        "help": "A unix timestamp representing an end datetime.",
    },
}
